from typing import Optional, List, Any


def __read_only_properties(*attrs):
    def class_rebuilder(cls):
        "The class decorator"
        class NewClass(cls):
            "This is the overwritten class"
            def __setattr__(self, name, value):
                if name not in attrs:
                    pass
                elif name not in self.__dict__:
                    pass
                else:
                    raise AttributeError("Can't modify {}".format(name))
                super().__setattr__(name, value)
        return NewClass
    return class_rebuilder


@__read_only_properties('files', 'mode')
class Tee:

    def __init__(self, *files: List[str], mode: str = 'w'):
        if mode not in ['w', 'a']:
            raise ValueError(f"Invalid mode '{mode}': select from ['w', 'a']")
        self.files = [open(f, mode) for f in files]
        self.mode = mode

    def __call__(self, *args: Any, sep: Optional[str] = None, end: Optional[str] = None) -> None:
        for f in self.files:
            print(*args, sep=sep, end=end, file=f, flush=True)
        print(*args, sep=sep, end=end)


def tee(*args: Any, sep: Optional[str] = None, end: Optional[str] = None, files: List[str] = [], mode: str = 'w') -> None:
    if mode not in ['w', 'a']:
            raise ValueError(f"Invalid mode '{mode}': select from ['w', 'a']")

    for fname in files:
        with open(fname, mode) as f:
            print(*args, sep=sep, end=end, file=f, flush=True)
    print(*args, sep=sep, end=end)


__all__ = ["Tee", "tee"]



from distutils.core import setup
setup(
  name = 'python-tee',
  packages = ['tee'],
  version = '0.0.5',
  license='MIT',
  description = '',
  author = 'Dante Biase',
  url = 'https://github.com/dante-biase/python-tee',
  download_url = 'https://github.com/dante-biase/python-tee/archive/v0.0.5.tar.gz',
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
  ],
)